@@ -1,17 +1,19 @@
-use rust_gpu_tools::opencl;
 use std::fmt::{self, Debug};
 use std::marker::PhantomData;
 use std::sync::{Arc, Mutex};
 
 use crate::error::{ClError, Error};
 use crate::poseidon::SimplePoseidonBatchHasher;
-#[cfg(feature = "opencl")]
+#[cfg(any(feature = "cuda", feature = "opencl"))]
 use crate::proteus::gpu::ClBatchHasher;
 #[cfg(feature = "futhark")]
 use crate::triton::{cl, gpu::GpuBatchHasher};
 use crate::{Arity, BatchHasher, Strength, DEFAULT_STRENGTH};
-use bellperson::bls::Fr;
+use blstrs::Scalar as Fr;
 use generic_array::GenericArray;
+#[cfg(feature = "futhark")]
+use rust_gpu_tools::opencl;
+use rust_gpu_tools::Device;
 
 #[cfg(feature = "futhark")]
 use triton::FutharkContext;
@@ -23,7 +25,7 @@ where
     Cpu(SimplePoseidonBatchHasher<A>),
     #[cfg(feature = "futhark")]
     OpenCl(GpuBatchHasher<A>),
-    #[cfg(feature = "opencl")]
+    #[cfg(any(feature = "cuda", feature = "opencl"))]
     OpenCl(ClBatchHasher<A>),
 }
 
@@ -56,33 +58,37 @@ where
     }
 
     /// Create a new GPU batcher for an arbitrarily picked device.
-    #[cfg(feature = "opencl")]
+    #[cfg(any(feature = "cuda", feature = "opencl"))]
     pub fn pick_gpu(max_batch_size: usize) -> Result<Self, Error> {
-        let all = opencl::Device::all();
-        let device = all.first().ok_or(Error::ClError(ClError::DeviceNotFound))?;
+        let device = *Device::all()
+            .first()
+            .ok_or(Error::ClError(ClError::DeviceNotFound))?;
         Self::new(device, max_batch_size)
     }
 
     #[cfg(feature = "futhark")]
     /// Create a new GPU batcher for a certain device.
-    pub fn new(device: &opencl::Device, max_batch_size: usize) -> Result<Self, Error> {
+    pub fn new(device: &Device, max_batch_size: usize) -> Result<Self, Error> {
         Self::with_strength(device, DEFAULT_STRENGTH, max_batch_size)
     }
 
-    #[cfg(feature = "opencl")]
+    #[cfg(any(feature = "cuda", feature = "opencl"))]
     /// Create a new GPU batcher for a certain device.
-    pub fn new(device: &opencl::Device, max_batch_size: usize) -> Result<Self, Error> {
+    pub fn new(device: &Device, max_batch_size: usize) -> Result<Self, Error> {
         Self::with_strength(device, DEFAULT_STRENGTH, max_batch_size)
     }
 
     #[cfg(feature = "futhark")]
     /// Create a new GPU batcher for a certain device with a specified strength.
     pub fn with_strength(
-        device: &opencl::Device,
+        device: &Device,
         strength: Strength,
         max_batch_size: usize,
     ) -> Result<Self, Error> {
-        let futhark_context = cl::futhark_context(&device)?;
+        let opencl_device = device
+            .opencl_device()
+            .ok_or(Error::ClError(ClError::DeviceNotFound))?;
+        let futhark_context = cl::futhark_context(&opencl_device)?;
         Ok(Self::OpenCl(GpuBatchHasher::<A>::new_with_strength(
             futhark_context,
             strength,
@@ -90,15 +96,15 @@ where
         )?))
     }
 
-    #[cfg(feature = "opencl")]
+    #[cfg(any(feature = "cuda", feature = "opencl"))]
     /// Create a new GPU batcher for a certain device with a specified strength.
     pub fn with_strength(
-        device: &opencl::Device,
+        device: &Device,
         strength: Strength,
         max_batch_size: usize,
     ) -> Result<Self, Error> {
         Ok(Self::OpenCl(ClBatchHasher::<A>::new_with_strength(
-            &device,
+            device,
             strength,
             max_batch_size,
         )?))
@@ -112,7 +118,7 @@ where
     fn hash(&mut self, preimages: &[GenericArray<Fr, A>]) -> Result<Vec<Fr>, Error> {
         match self {
             Batcher::Cpu(batcher) => batcher.hash(preimages),
-            #[cfg(any(feature = "futhark", feature = "opencl"))]
+            #[cfg(any(feature = "futhark", feature = "cuda", feature = "opencl"))]
             Batcher::OpenCl(batcher) => batcher.hash(preimages),
         }
     }
@@ -120,7 +126,7 @@ where
     fn max_batch_size(&self) -> usize {
         match self {
             Batcher::Cpu(batcher) => batcher.max_batch_size(),
-            #[cfg(any(feature = "futhark", feature = "opencl"))]
+            #[cfg(any(feature = "futhark", feature = "cuda", feature = "opencl"))]
             Batcher::OpenCl(batcher) => batcher.max_batch_size(),
         }
     }
